@@ -26,6 +26,7 @@
 
 #include <cmake.h>
 #include <Eval.h>
+#include <DOM.h>
 #include <map>
 #include <time.h>
 #include <Context.h>
@@ -34,8 +35,6 @@
 #include <shared.h>
 #include <format.h>
 
-extern Task& contextTask;
-
 ////////////////////////////////////////////////////////////////////////////////
 // Supported operators, borrowed from C++, particularly the precedence.
 // Note: table is sorted by length of operator string, so searches match
@@ -101,6 +100,19 @@ static bool namedConstants (const std::string& name, Variant& value)
   return true;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+// Support for evaluating DOM references (add with `e.AddSource(domSource)`)
+bool domSource (const std::string& identifier, Variant& value)
+{
+  if (getDOM (identifier, Context::getContext ().currentTask, value))
+  {
+    value.source (identifier);
+    return true;
+  }
+
+  return false;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 Eval::Eval ()
 {
@@ -278,6 +290,8 @@ void Eval::evaluatePostfixStack (
       Variant left = values.back ();
       values.pop_back ();
 
+      auto contextTask = Context::getContext ().currentTask;
+
       // Ordering these by anticipation frequency of use is a good idea.
       Variant result;
            if (token.first == "and")      result = left && right;
@@ -299,10 +313,14 @@ void Eval::evaluatePostfixStack (
       else if (token.first == "^")        result = left ^ right;
       else if (token.first == "%")        result = left % right;
       else if (token.first == "xor")      result = left.operator_xor (right);
-      else if (token.first == "~")        result = left.operator_match (right, contextTask);
-      else if (token.first == "!~")       result = left.operator_nomatch (right, contextTask);
-      else if (token.first == "_hastag_") result = left.operator_hastag (right, contextTask);
-      else if (token.first == "_notag_")  result = left.operator_notag (right, contextTask);
+      else if (contextTask) {
+             if (token.first == "~")        result = left.operator_match (right, *contextTask);
+        else if (token.first == "!~")       result = left.operator_nomatch (right, *contextTask);
+        else if (token.first == "_hastag_") result = left.operator_hastag (right, *contextTask);
+        else if (token.first == "_notag_")  result = left.operator_notag (right, *contextTask);
+        else
+          throw format ("Unsupported operator '{1}'.", token.first);
+      }
       else
         throw format ("Unsupported operator '{1}'.", token.first);
 
