@@ -60,8 +60,6 @@
 
 #define APPROACHING_INFINITY 1000   // Close enough.  This isn't rocket surgery.
 
-extern Task& contextTask;
-
 static const float epsilon = 0.000001;
 #endif
 
@@ -114,6 +112,12 @@ bool Task::operator== (const Task& other)
   return true;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+bool Task::operator!= (const Task& other)
+{
+  return !(*this == other);
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 Task::Task (const std::string& input)
 {
@@ -363,6 +367,14 @@ Task::dateState Task::getDateState (const std::string& name) const
   return dateNotDue;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+// An empty task is typically a "dummy", such as in DOM evaluation, which may or
+// may not occur in the context of a task.
+bool Task::is_empty () const
+{
+  return data.size () == 0;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // Ready means pending, not blocked and either not scheduled or scheduled before
 // now.
@@ -765,7 +777,25 @@ void Task::parseJSON (const json::object* root_obj)
       else if (i.first == "depends" && i.second->type() == json::j_string)
       {
         auto deps = (json::string*)i.second;
-        auto uuids = split (deps->_data, ',');
+
+        // Fix for issue#2689: taskserver sometimes encodes the depends
+        // property as a string of the format `[\"uuid\",\"uuid\"]`
+        // The string includes the backslash-escaped `"` characters, making
+        // it invalid JSON.  Since we know the characters we're looking for,
+        // we'll just filter out everything else.
+        std::string deps_str = deps->_data;
+        if (deps_str.front () == '[' && deps_str.back () == ']') {
+          std::string filtered;
+		  for (auto &c: deps_str) {
+			if ((c >= '0' && c <= '9') ||
+				(c >= 'a' && c <= 'f') ||
+				c == ',' || c == '-') {
+			  filtered.push_back(c);
+			}
+		  }
+		  deps_str = filtered;
+        }
+        auto uuids = split (deps_str, ',');
 
         for (const auto& uuid : uuids)
           addDependency (uuid);
@@ -1545,7 +1575,7 @@ bool Task::isAnnotationAttr(const std::string& attr)
 #ifdef PRODUCT_TASKWARRIOR
 ////////////////////////////////////////////////////////////////////////////////
 // A UDA Orphan is an attribute that is not represented in context.columns.
-std::vector <std::string> Task::getUDAOrphanUUIDs () const
+std::vector <std::string> Task::getUDAOrphans () const
 {
   std::vector <std::string> orphans;
   for (auto& it : data)
@@ -2257,6 +2287,10 @@ void Task::modify (modType type, bool text_required /* = false */)
 {
   std::string label = "  [1;37;43mMODIFICATION[0m ";
 
+  // while reading the parse tree, consider DOM references in the context of
+  // this task
+  auto currentTask = Context::getContext ().withCurrentTask(this);
+
   // Need this for later comparison.
   auto originalStatus = getStatus ();
 
@@ -2276,6 +2310,19 @@ void Task::modify (modType type, bool text_required /* = false */)
             value == "''"   ||
             value == "\"\"")
         {
+          // Special case: Handle bulk removal of 'tags' and 'depends" virtual
+          // attributes
+          if (name == "depends")
+          {
+            for (auto dep: getDependencyUUIDs ())
+              removeDependency(dep);
+          }
+          else if (name == "tags")
+          {
+            for (auto tag: getTags ())
+              removeTag(tag);
+          }
+
           // ::composeF4 will skip if the value is blank, but the presence of
           // the attribute will prevent ::validate from applying defaults.
           if ((has (name) && get (name) != "") ||
@@ -2404,7 +2451,8 @@ void Task::modify (modType type, bool text_required /* = false */)
 #endif
 
 ////////////////////////////////////////////////////////////////////////////////
-// Compare this task to another and summarize the differences for display
+// Compare this task to another and summarize the differences for display, in
+// the future tense ("Foo will be set to ..").
 std::string Task::diff (const Task& after) const
 {
   // Attributes are all there is, so figure the different attribute names
@@ -2649,3 +2697,206 @@ std::string Task::diffForInfo (
 }
 
 ////////////////////////////////////////////////////////////////////////////////
+// Similar to diff, but formatted as a side-by-side table for an Undo preview
+Table Task::diffForUndoSide (
+  const Task& after) const
+{
+  // Set the colors.
+  Color color_red   (Context::getContext ().color () ? Context::getContext ().config.get ("color.undo.before") : "");
+  Color color_green (Context::getContext ().color () ? Context::getContext ().config.get ("color.undo.after") : "");
+
+  // Attributes are all there is, so figure the different attribute names
+  // between before and after.
+  Table view;
+  view.width (Context::getContext ().getWidth ());
+  view.intraPadding (2);
+  view.add ("");
+  view.add ("Prior Values");
+  view.add ("Current Values");
+  setHeaderUnderline (view);
+
+  if (!is_empty ())
+  {
+    const Task &before = *this;
+
+    std::vector <std::string> beforeAtts;
+    for (auto& att : before.data)
+      beforeAtts.push_back (att.first);
+
+    std::vector <std::string> afterAtts;
+    for (auto& att : after.data)
+      afterAtts.push_back (att.first);
+
+    std::vector <std::string> beforeOnly;
+    std::vector <std::string> afterOnly;
+    listDiff (beforeAtts, afterAtts, beforeOnly, afterOnly);
+
+    int row;
+    for (auto& name : beforeOnly)
+    {
+      row = view.addRow ();
+      view.set (row, 0, name);
+      view.set (row, 1, renderAttribute (name, before.get (name)), color_red);
+    }
+
+    for (auto& att : before.data)
+    {
+      std::string priorValue   = before.get (att.first);
+      std::string currentValue = after.get  (att.first);
+
+      if (currentValue != "")
+      {
+        row = view.addRow ();
+        view.set (row, 0, att.first);
+        view.set (row, 1, renderAttribute (att.first, priorValue),
+                  (priorValue != currentValue ? color_red : Color ()));
+        view.set (row, 2, renderAttribute (att.first, currentValue),
+                  (priorValue != currentValue ? color_green : Color ()));
+      }
+    }
+
+    for (auto& name : afterOnly)
+    {
+      row = view.addRow ();
+      view.set (row, 0, name);
+      view.set (row, 2, renderAttribute (name, after.get (name)), color_green);
+    }
+  }
+  else
+  {
+    int row;
+    for (auto& att : after.data)
+    {
+      row = view.addRow ();
+      view.set (row, 0, att.first);
+      view.set (row, 2, renderAttribute (att.first, after.get (att.first)), color_green);
+    }
+  }
+
+  return view;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// Similar to diff, but formatted as a diff for an Undo preview
+Table Task::diffForUndoPatch (
+  const Task& after,
+  const Datetime& lastChange) const
+{
+  // This style looks like this:
+  //  --- before    2009-07-04 00:00:25.000000000 +0200
+  //  +++ after    2009-07-04 00:00:45.000000000 +0200
+  //
+  // - name: old           // att deleted
+  // + name:
+  //
+  // - name: old           // att changed
+  // + name: new
+  //
+  // - name:
+  // + name: new           // att added
+  //
+
+  // Set the colors.
+  Color color_red   (Context::getContext ().color () ? Context::getContext ().config.get ("color.undo.before") : "");
+  Color color_green (Context::getContext ().color () ? Context::getContext ().config.get ("color.undo.after") : "");
+
+  const Task &before = *this;
+
+  // Generate table header.
+  Table view;
+  view.width (Context::getContext ().getWidth ());
+  view.intraPadding (2);
+  view.add ("");
+  view.add ("");
+
+  int row = view.addRow ();
+  view.set (row, 0, "--- previous state", color_red);
+  view.set (row, 1, "Undo will restore this state", color_red);
+
+  row = view.addRow ();
+  view.set (row, 0, "+++ current state ", color_green);
+  view.set (row, 1, format ("Change made {1}",
+                            lastChange.toString (Context::getContext ().config.get ("dateformat"))),
+                    color_green);
+
+  view.addRow ();
+
+  // Add rows to table showing diffs.
+  std::vector <std::string> all = Context::getContext ().getColumns ();
+
+  // Now factor in the annotation attributes.
+  for (auto& it : before.data)
+    if (it.first.substr (0, 11) == "annotation_")
+      all.push_back (it.first);
+
+  for (auto& it : after.data)
+    if (it.first.substr (0, 11) == "annotation_")
+      all.push_back (it.first);
+
+  // Now render all the attributes.
+  std::sort (all.begin (), all.end ());
+
+  std::string before_att;
+  std::string after_att;
+  std::string last_att;
+  for (auto& a : all)
+  {
+    if (a != last_att)  // Skip duplicates.
+    {
+      last_att = a;
+
+      before_att = before.get (a);
+      after_att  = after.get (a);
+
+      // Don't report different uuid.
+      // Show nothing if values are the unchanged.
+      if (a == "uuid" ||
+          before_att == after_att)
+      {
+        // Show nothing - no point displaying that which did not change.
+
+        // row = view.addRow ();
+        // view.set (row, 0, *a + ":");
+        // view.set (row, 1, before_att);
+      }
+
+      // Attribute deleted.
+      else if (before_att != "" && after_att == "")
+      {
+        row = view.addRow ();
+        view.set (row, 0, '-' + a + ':', color_red);
+        view.set (row, 1, before_att, color_red);
+
+        row = view.addRow ();
+        view.set (row, 0, '+' + a + ':', color_green);
+      }
+
+      // Attribute added.
+      else if (before_att == "" && after_att != "")
+      {
+        row = view.addRow ();
+        view.set (row, 0, '-' + a + ':', color_red);
+
+        row = view.addRow ();
+        view.set (row, 0, '+' + a + ':', color_green);
+        view.set (row, 1, after_att, color_green);
+      }
+
+      // Attribute changed.
+      else
+      {
+        row = view.addRow ();
+        view.set (row, 0, '-' + a + ':', color_red);
+        view.set (row, 1, before_att, color_red);
+
+        row = view.addRow ();
+        view.set (row, 0, '+' + a + ':', color_green);
+        view.set (row, 1, after_att, color_green);
+      }
+    }
+  }
+
+  return view;
+}
+
+////////////////////////////////////////////////////////////////////////////////
