@@ -36,8 +36,6 @@
 #include <utf8.h>
 #include <util.h>
 
-extern Task& contextTask;
-
 ////////////////////////////////////////////////////////////////////////////////
 ColumnProject::ColumnProject ()
 {
@@ -121,7 +119,6 @@ void ColumnProject::modify (Task& task, const std::string& value)
     {
       Eval e;
       e.addSource (domSource);
-      contextTask = task;
 
       Variant v;
       e.evaluateInfixExpression (value, v);
