@@ -36,8 +36,6 @@
 #include <format.h>
 #include <utf8.h>
 
-extern Task& contextTask;
-
 ////////////////////////////////////////////////////////////////////////////////
 ColumnRecur::ColumnRecur ()
 {
@@ -108,7 +106,6 @@ void ColumnRecur::modify (Task& task, const std::string& value)
   {
     Eval e;
     e.addSource (domSource);
-    contextTask = task;
     e.evaluateInfixExpression (value, evaluatedValue);
   }
 
