@@ -36,8 +36,6 @@
 #include <utf8.h>
 #include <main.h>
 
-extern Task& contextTask;
-
 ////////////////////////////////////////////////////////////////////////////////
 ColumnTags::ColumnTags ()
 {
@@ -162,7 +160,6 @@ void ColumnTags::modify (Task& task, const std::string& value)
   {
     Eval e;
     e.addSource (domSource);
-    contextTask = task;
 
     Variant v;
     e.evaluateInfixExpression (value, v);
