@@ -32,8 +32,6 @@
 #include <Filter.h>
 #include <format.h>
 
-extern Task& contextTask;
-
 ////////////////////////////////////////////////////////////////////////////////
 ColumnTypeNumeric::ColumnTypeNumeric ()
 {
@@ -55,7 +53,6 @@ void ColumnTypeNumeric::modify (Task& task, const std::string& value)
   {
     Eval e;
     e.addSource (domSource);
-    contextTask = task;
     e.evaluateInfixExpression (value, evaluatedValue);
   }
 
