@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2006 - 2021, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2021, Tomas Babej, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -70,6 +70,9 @@ int CmdAnnotate::execute (std::string&)
   // Accumulated project change notifications.
   std::map <std::string, std::string> projectChanges;
 
+  if(filtered.size() > 1) {
+    feedback_affected("This command will alter {1} tasks.", filtered.size());
+  }
   for (auto& task : filtered)
   {
     Task before (task);
@@ -81,7 +84,7 @@ int CmdAnnotate::execute (std::string&)
 
     task.modify (Task::modAnnotate, true);
 
-    if (permission (taskDifferences (before, task) + question, filtered.size ()))
+    if (permission (before.diff (task) + question, filtered.size ()))
     {
       Context::getContext ().tdb2.modify (task);
       ++count;
