@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2006 - 2019, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2021, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -276,6 +276,12 @@ Color::Color (color_id fg, color_id bg, bool underline, bool bold, bool bright)
   }
 }
 
+////////////////////////////////////////////////////////////////////////////////
+bool Color::operator== (const Color& other) const
+{
+  return int(_value) == int(other);
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 Color::operator std::string () const
 {
@@ -315,7 +321,7 @@ Color::operator int () const
 // other take precedence.
 void Color::blend (const Color& other)
 {
-  if (!other.nontrivial ())
+  if (!other.nontrivial () || (*this) == other)
     return;
 
   Color c (other);
