@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2013 - 2019, Paul Beckingham, Federico Hernandez.
+// Copyright 2013 - 2021, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -37,7 +37,7 @@
 static const std::string uuid_pattern = "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx";
 static const unsigned int uuid_min_length = 8;
 
-std::string Lexer::dateFormat = "";
+std::string Lexer::dateFormat;
 
 ////////////////////////////////////////////////////////////////////////////////
 Lexer::Lexer (const std::string& text)
