@@ -29,6 +29,7 @@
 #include <new>
 #include <cstring>
 #include <Context.h>
+#include <regex>
 
 ////////////////////////////////////////////////////////////////////////////////
 int main (int argc, const char** argv)
@@ -64,6 +65,11 @@ int main (int argc, const char** argv)
       status = -3;
     }
 
+    catch (const std::regex_error& e)
+    {
+      std::cout << "regex_error caught: " << e.what() << '\n';
+    }
+
     catch (...)
     {
       std::cerr << "Unknown error. Please report.\n";
